// the EditorManager provides methods to manage (add/remove) editors on a page. 
// There is only allowed (at this time) to be one active editor per page. The current active
// will be deactivated before activating another.
EditorManager = {};
EditorManager.Manager = Class.create();	
EditorManager.Manager.prototype = {
	initialize: function() {
		Event.observe(document, 'click', this.globalClickHandler.bind(this), false);
	},
	activeEditor : null,
	editors : new $H(),
	toolbarClick: false,
	globalClickHandler: function(event) {
		this.unedit(this.activeEditor);
	},
	getEditor : function(elementId, options) {
		var editor = this.editors[elementId];
		if( editor == null ) {
			var onBlurCallback = function() {};
			
			// NOTE: i had to change tiny_mce_src.js to make the onBlurCallback work,
			// if you upgrade tiny_mce_src.js, you'll need to re-add the code.
			if( options['uneditOnBlur'] || options['dynamic'] ) {
				onBlurCallback = function() {	
					this.uneditById(elementId);
				}
			}
			editor = new EditorManager.Editor(elementId, onBlurCallback.bind(this), options);
			// this will show the "DONE" control when editor is showing
			editor.showCallback = function(elId, showCtrl) {
				var el = $('edit_' + elId);
				if( el != null ) {
					el.innerHTML = 'DONE';
				}
			}
			// this will show the "EDIT" control when editor is hiding
			editor.hideCallback = function(elId, showCtrl) {
				var el = $('edit_' + elId);
				if( el != null ) {
					el.innerHTML = 'EDIT';
				} 
			}
			this.editors[elementId] = editor;

		}
		if( editor.isDynamic() ) {
			this.registerForEvents(elementId);				    
		}
		return editor;
	},
	registerForEvents: function(elementId) {
		Logger.logDebug('registerForEvents');
		if( elementId == null || $(elementId) == null ) return;
		Event.observe(elementId, 'click', this.observerCallback, false);
		
		// if the only child of the element is a div, register with that one as well
		var children = $(elementId).childNodes;
		if( children != null && children.length > 0 ) {
			var child = children[0];
			if( child.nodeName == 'div' || child.nodeName == 'DIV' ) {
				Event.observe(child, 'click', this.observerCallback, false);				
			}
		}
	},
	observerCallback: function(event){
		Logger.logDebug('observerCallback:');
    	var element = event.target || event.srcElement;
    	if( element == null ) return;
    	try {
			Logger.logDebug('id='+element.id+', element='+element);
	        while(!editorManager.editById(element.id)) {
				// try the parent node
				element = element.parentNode;
				if( element == null ) break;
			}
    	} catch(ex) {

    	}
	},
	unregisterForEvents: function(elementId) {
		Logger.logDebug('unregisterForEvents: '+elementId);
		if( elementId == null )	return;
		Event.stopObserving(elementId, 'click', this.observerCallback);
		
		// if the only child of the element is a div, unregister with that one as well
		var children = $(elementId).children;
		if( children != null && children.length == 1) {
			var child = children[0];
			if( child.name == 'div' || child.name == 'DIV' ) {
				Event.stopObserving(child, 'click', this.observerCallback);				
			}
		}

	},
	toggleEdit : function(elementId, options) {
		var editor = this.getEditor(elementId, options || {});
		if(editor.hidden) {
			this.edit(editor);
		} else {
			this.unedit(editor);	
		}
	},
    editById : function(elementId) {	
		Logger.logDebug('editById:'+elementId);
		if( elementId == null || elementId == '') return false;
		this.unregisterForEvents(elementId);	
		
   		var editor = this.editors[elementId];
		if( editor != null ) {
			return this.edit(editor);
		}
		return false;
    },
    edit : function(editor) {
		if(this.activeEditor != null) {
		Logger.logDebug('entering edit');
			this.unedit(this.activeEditor);
		}

		this.activeEditor = editor;
		
		editor.show();
		
		var toolbar = document.getElementsByClassName('mceToolbar')[0];
		Event.observe(toolbar, 'mousedown', (function(event) {
					editorManager.toolbarClick = true;
					Event.stop(event);
					setTimeout( function() { editorManager.toolbarClick = false; }, 1000);					
					return false;
   				}).bind(this),
			false
		);
	 },
	 uneditById : function(elementId) {
		var editor = this.editors[elementId];
		if( editor != null && (!tinyMCE.isMSIE || !this.toolbarClick)) {
			this.unedit(editor);
		}
		this.toolbarClick = false;
	 },
	 unedit : function(editor) {
		Logger.logDebug('entering unedit: activeEditor='+this.activeEditor+', editor='+editor);
	 	if( editor == null || this.toolbarClick || !editor.isDynamic() ) return;
		this.activeEditor = null;

		editor.hide();
		if( !editor.isInitialValue() ) {
			editor.updateToChangedClass();
			
			if( editor.originalFgColor != null ) {
				$(editor.elementId).style.color = editor.originalFgColor;
			}
		}
		setTimeout( (function() {
				this.registerForEvents(editor.elementId);
			}).bind(this), 10);
	 }, 
	 contentChanged: function(inst) {
	 	var editorId = inst.formTargetElementId;
	 	var editor = this.editors[editorId];

	 	
	 	//propagate the event to the editor itself
		if( editor != null ) {
	 		editor.contentChanged(inst);
		}
	 }
}

// Editor class, encapsulates and hides the specific rich editor implemenation (in this case tinyMCE)
EditorManager.Editor = {};
EditorManager.Editor = Class.create();	
EditorManager.Editor.prototype = {
	firstFocus: true,
	controls: "link,bold,italic,underline,justifyleft,justifycenter,justifyright,forecolor,backcolor",
	baseOptions: new $H({
		 theme : "advanced",
		 plugins : "inlinepopups",
		 theme_advanced_layout_manager : "RowLayout", 
		 relative_urls : false,
		 onblur_callback : this.onBlurCallback,
		 safari_warning : false
	}),
	initialize: function(elementId, onBlurCallback, options) {
//		if( BrowserInfo.isIE() ) {
			this.baseOptions['plugins'] = '';
//		}
		this.elementId = elementId;
		this.hidden = true;
		this.options = options || {};
		
		if( onBlurCallback != null ) {
			this.onBlurCallback = onBlurCallback;
		} else {
			this.onBlurCallback = function() {};
		}
		if( this.isDynamic() ) {				
			this.setupInitialValue();
		}
	},
	isInitialValue: function() {
		var initialValue = this.options['initialValue'];
		var value = $(this.elementId).innerHTML;
		return( (initialValue != null) && (value == '' || value == initialValue) );
	},
	setupInitialValue: function() {
		if( $(this.elementId) == null ) return;
		if( this.isInitialValue() ) {
			Element.update(this.elementId, this.options['initialValue']);
			this.originalFgColor = $(this.elementId).style.color;
		}
	},
	updateToChangedClass: function() {
		var changedClass = this.options['changedClass'];		

		if( changedClass != null ) { 
			Element.classNames(this.elementId).set(changedClass);
		}
	},
	show : function() {
		this.init();
		tinyMCE.execCommand('mceAddControl', null, this.elementId);
		this.hidden = false;		
		tinyMCE.execCommand('mceFocus', null, this.elementId);		
		this.showCallback(this.elementId);
		// select all
		this.selectAll();
		this.firstFocus = false;
		
		this.sendTrackingInfo();
	},
	sendTrackingInfo : function() {
		if( !this.sentTrackingInfo ) {
			 Tracker.sendTrackingEvent(this.elementId);
			 this.sentTrackingInfo = true;
		}	
	},
	isDynamic: function() {
		return (this.options['dynamic'] == true);	
	},
	hide : function() {
		tinyMCE.execCommand('mceRemoveControl', null, this.elementId);	
		this.hideCallback(this.elementId);
		this.hidden = true;				
	},
	selectAll: function() {
		if( !this.firstFocus ) return;
		setTimeout( function() {
			if( tinyMCE.selectedInstance != null ) {
				tinyMCE.execCommand('selectall', false, this.elementId);
			}
		}, 100);
	},
	init : function() {
		var allOptions = new $H().merge(this.baseOptions);
		 allOptions = allOptions.merge( {
		     theme_advanced_containers : "top1,mceEditor",
		     theme_advanced_containers_default_class : "mceToolbar",
		     theme_advanced_containers_default_align : "center",
		     theme_advanced_container_top1 : this.controls,
		     force_br_newlines : true,
		     force_p_newlines : false,
		     convert_newlines_to_brs : true,
		     relative_urls : false
	     } );

		var height = this.options['height'];
		if( height != null ) allOptions['height'] = height;
		
		var width = this.options['width'];
		if( width != null ) allOptions['width'] = width;
		
		if( this.onBlurCallback != null ) allOptions['onblur_callback'] = this.onBlurCallback;		

		allOptions['onchange_callback'] = 'editorManager.contentChanged';
		
		this.characterCounterDivId = this.options['characterCounterDivId'];
		this.maxChars = this.options['maxChars'] || -1;		
		
		tinyMCE.init(allOptions);
	},
    contentChanged: function(inst) {
    	if( this.maxChars == -1 ) return;
		var body = inst.getBody().innerHTML || '';
		var len = body.length;
		var lenLeft = this.maxChars - len;
	    if( lenLeft <= 0 ) {
			body = body.substring(0, this.maxChars-1);
			inst.setContent(body);
			lenLeft = 0;
		}
		
    	if( this.characterCounterDivId == null ) return;
		Element.update( this.characterCounterDivId, lenLeft+'' );
    },
    getValue: function() {
    	if( this.hidden ) {
    		return $(this.elementId).innerHTML;
    	} else {
			return tinyMCE.getContent();
		}
    },
    setValue: function(value) {
    	if( this.hidden ) {
    		return Element.update(this.elementId, value);
    	} else {
			return tinyMCE.setContent(value);
		}
    },
	// these are just stubs, meant to be overridden
	showCallback : function(elementId, showControls) {},
	hideCallback : function(elementId, showControls) {}
}


// set up a (page) singleton for the editorManager
var editorManager = new EditorManager.Manager();
new EditorManager.Editor('', false).init();

